from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

class DjangoInitialsAvatarConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_initials_avatar'
    verbose_name = _("Initials Avatar")



class SVGAvatar:

    @classmethod
    def name_to_letters(cls, name: str, length: int) -> str:
        name_parts = list(map(lambda part: str(part)[0], name.split(' ')))

        if length > 1:
            new_length = length - 2

            if new_length > 0:
                first_letter = name_parts[0]
                last_letter = name_parts[-1]
                middle_letters = name_parts[1:new_length + 1]
                letters = ''.join([first_letter] + middle_letters + [last_letter])
            else:
                letters = ''.join([name_parts[0], name_parts[-1]])

            return letters
        else:
            return name_parts[0]

    @classmethod
    def render(cls, name: str, background: str, color: str, text_length: int, avatar_width: int, avatar_height: int,
               font_size: int,
               avatar_rounded: bool, text_capitalize: bool, text_lowercase: bool, text_bold: bool) -> str:
        letter_name = cls.name_to_letters(name, length=text_length)
        if text_capitalize:
            letter_name = letter_name.upper()
        elif text_lowercase:
            letter_name = letter_name.lower()

        style = "font-weight:700;" if text_bold else ""
        return f'<svg style="{style}" width="{avatar_width}px" height="{avatar_height}px" ' \
               f'xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><defs>' \
               f'<style type="text/css">' \
               '@font-face {font-family: "montserratbold";src: url("https://cdn.oxro.io/fonts/montserrat-bold-webfont.woff2") format("woff2"),' \
               'url("https://cdn.oxro.io/fonts/montserrat-bold-webfont.woff") format("woff");' \
               'font-weight: normal;font-style: normal;}</style></defs>' \
               f'<rect x="0" y="0" width="500" height="500" rx="{50 if avatar_rounded else 0}" ' \
               f'style="fill:{background}"/>' \
               f'<text x="50%" y="50%" dy=".1em" fill="{color}" ' \
               f'text-anchor="middle" dominant-baseline="middle" ' \
               f'style="font-family: &quot;Montserrat&quot;, sans-serif; font-size: {font_size}px; line-height: 1">' \
               f'{letter_name}</text>' \
               f'</svg>'


from django.conf import settings

BG_COLORS = getattr(settings, 'INITIALS_AVATAR_BG_COLORS', [
    "#E284B3", "#FFED8B", "#681313", "#F3C1C6", "#735372", "#009975", "#FFBD39", "#B1E8ED", "#52437B", "#F76262",
    "#216583", "#293462", "#DD9D52", "#936B93", "#6DD38D", "#888888", "#6F8190", "#BCA0F0", "#AAF4DD", "#96C2ED",
    "#3593CE", "#5EE2CD", "#96366E", "#E38080"
])
TEXT_COLOR = getattr(settings, 'INITIALS_AVATAR_TEXT_COLOR', '#fff')
TEXT_LENGTH = getattr(settings, 'INITIALS_AVATAR_TEXT_LENGTH', 2)
WIDTH = getattr(settings, 'INITIALS_AVATAR_WIDTH', 500)
HEIGHT = getattr(settings, 'INITIALS_AVATAR_HEIGHT', 500)
FONT_SIZE = getattr(settings, 'INITIALS_AVATAR_FONT_SIZE', 250)
ROUNDED = getattr(settings, 'INITIALS_AVATAR_ROUNDED', False)
CAPITALIZE = getattr(settings, 'INITIALS_AVATAR_CAPITALIZE', False)
LOWERCASE = getattr(settings, 'INITIALS_AVATAR_LOWERCASE', False)
BOLD = getattr(settings, 'INITIALS_AVATAR_BOLD', True)
CACHE_TIMEOUT = getattr(settings, 'INITIALS_AVATAR_CACHE_TIMEOUT', 60 * 60)


from django import template
from django.urls import reverse
from urllib.parse import urlencode

register = template.Library()


@register.simple_tag
def render_initials_avatar(name, **kwargs):
    kwargs['name'] = name
    return f"%s?{urlencode(kwargs)}" % reverse('initials-avatar-svg')



import random
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from django.views.generic import View
from django.http import HttpResponse
from django_initials_avatar.app_settings import (CACHE_TIMEOUT, BG_COLORS,
                                                 TEXT_COLOR, TEXT_LENGTH, WIDTH, HEIGHT, FONT_SIZE, ROUNDED,
                                                 CAPITALIZE, LOWERCASE, BOLD)
from django_initials_avatar.utils.svg_avatar import SVGAvatar


class BaseAvatarView(View):

    @property
    def name(self) -> str:
        return str(self.request.GET.get('name'))

    @property
    def background(self) -> str:
        return str(self.request.GET.get('background', random.choice(BG_COLORS)))

    @property
    def color(self) -> str:
        return str(self.request.GET.get('color', TEXT_COLOR))

    @property
    def text_length(self) -> int:
        return int(self.request.GET.get('length', TEXT_LENGTH))

    @property
    def avatar_width(self) -> int:
        return int(self.request.GET.get('width', WIDTH))

    @property
    def avatar_height(self) -> int:
        return int(self.request.GET.get('height', HEIGHT))

    @property
    def font_size(self) -> int:
        return int(self.request.GET.get('size', FONT_SIZE))

    @property
    def avatar_rounded(self) -> bool:
        return bool(self.request.GET.get('rounded', ROUNDED))

    @property
    def text_capitalize(self) -> bool:
        return bool(self.request.GET.get('capitalize', CAPITALIZE))

    @property
    def text_lowercase(self) -> bool:
        return bool(self.request.GET.get('lowercase', LOWERCASE))

    @property
    def text_bold(self) -> bool:
        return bool(self.request.GET.get('bold', BOLD))


@method_decorator(cache_page(CACHE_TIMEOUT), name='dispatch')
class SVGInitialsAvatarView(BaseAvatarView):
    def get(self, request, *args, **kwargs):
        return HttpResponse(
            SVGAvatar.render(
                name=self.name,
                background=self.background,
                color=self.color,
                text_length=self.text_length,
                avatar_width=self.avatar_width,
                avatar_height=self.avatar_height,
                font_size=self.font_size,
                avatar_rounded=self.avatar_rounded,
                text_capitalize=self.text_capitalize,
                text_lowercase=self.text_lowercase,
                text_bold=self.text_bold
            ),
            content_type='image/svg+xml'
        )


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django_initials_avatar",
    version="0.0.1",
    author="Henrique da Silva Santos",
    author_email="henrique.santos@4u360.com.br",
    description="A ridiculously simple avatar generator with initials from names.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/KoterIA/FacebookCloudAPI",
    project_urls={
        "Bug Tracker": "https://github.com/KoterIA/FacebookCloudAPI/issues",
        "Repository": "https://github.com/KoterIA/FacebookCloudAPI",
    },
    install_requires=[
        'Django>=2.2.0',
    ],
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3"
    ],
    packages=setuptools.find_packages(exclude=("tests",)),
    include_package_data=True,
    python_requires=">=3.8"
)
